#!/usr/bin/perl -w
#
# SuperMRP.pl v1.2.1
# Last modified November 14, 2005 10:17
# (c) Olaf R.P. Bininda-Emonds
#
# Generates:
#   A nexus-formatted file (MRPmatrix.nex) containing the matrix
#   representations of the tree descriptions in a nexus treefile
#
# Usage: SuperMRP.pl [-b] [-f<filename>] [-n] [-p] [-r] [-s] [-t] [-v] [-h]
#   options: -b = process first tree block only
#            -f<filename> = file containing trees in NEXUS format
#                           (default = treefile.tre)
#            -n = ignore tree names in treefile
#            -p = use Purvis MRP coding (default = standard MRP coding)
#			 -r = source trees to be rooted unless specified otherwise in treefile
#				  (default = unrooted)
#            -s = use branch lengths (if present) as nodal supports
#                 Note: support for nodes without such information taken as average
#                 of support of all other nodes
#            -t = datafile in transposed format
#            -v = verbose output
#            -h = print this message and quit

use strict;

# Set user-input defaults and associated parameters
	# I/O parameters
		my $block = 1;
			my $treeBlock = 0;
		my $treeFile = "treefile.tre";
		my $keepName = 1;
		
	# Tree parameters
		my (%treeName, %treeDescription, %rootingStatus);
		my $unrootedFlag = 0;
		my $defaultRooting = 0;	# Default is for trees to be unrooted
		my $SupportFlag = 0;
		my @allTaxa;

	# MRP options
		my $purvis = 0;
		my $MRPoutgroup;
		my $support = 0;
		my $transpose = 0;
		my $STsupport = 0;
	
	# Miscellaneous
		my $verbose = 0;
		my $debug = 0;
		my $version = "1.2.1";

for (my $i = 0; $i <= $#ARGV; $i++)
	{
	if ($ARGV[$i] eq "-b")
		{
		$block = 0;
		}
	elsif ($ARGV[$i] =~ /-f([\w+\.?\w+?]*)/)
		{
		$treeFile = $1;
		die "ERROR: Treefile $treeFile does not exist.\n" unless (-e $treeFile);
		}
	elsif ($ARGV[$i] eq "-n")
		{
		$keepName = 0;
		}
	elsif ($ARGV[$i] eq "-p")
		{
		$purvis = 1;
		}
	elsif ($ARGV[$i] eq "-r")
		{
		$defaultRooting = 1;
		}
	elsif ($ARGV[$i] eq "-s")
		{
		$support = 1;
		}
	elsif ($ARGV[$i] eq "-t")
		{
		$transpose = 1;
		}
	elsif ($ARGV[$i] =~ /-v/)
		{
		$verbose = 1;
		}
	elsif ($ARGV[$i] eq "-x")
		{
		$STsupport = 1;
		$verbose = $debug = 0;
		}
	elsif ($ARGV[$i] eq "-h")
		{
		print "Usage: SuperMRP.pl [-b] [-f<filename>] [-n] [-p] [-r] [-s] [-t] [-v] [-h]\n";
		print "Version: $version\n";
		print "Options: -b = process first tree block only\n";
		print "         -f<filename> = file containing trees in NEXUS format\n";
		print "                        (default = treefile.tre)\n";
		print "         -n = ignore tree names in treefile\n";
		print "         -p = use Purvis MRP coding (default = standard MRP coding)\n";
		print "         -r = source trees to be rooted unless specified otherwise in treefile\n";
		print "              (default = unrooted)\n";
		print "         -s = use branch lengths (if present) as nodal supports\n";
		print "              Note: support for nodes without such information taken as average\n";
		print "                    of support of all other nodes\n";
		print "         -t = datafile in transposed format\n";
		print "         -v = verbose output\n";
		print "         -h = print this message and quit\n";
		print "\nCitation: Bininda-Emonds, O.R.P., R.M.D. Beck, and A. Purvis. 2005. Getting to the roots of matrix representation. Systematic Biology 54(4):668672.\n";
		exit(0);
		}
	else
		{
		print "Don't understand argument: $ARGV[$i]\n";
		print "Usage: SuperMRP.pl [-b] [-f<filename>] [-n] [-p] [-r] [-s] [-t] [-v] [-h]\n";
		print "Version: $version\n";
		exit(1); 
		}
	}

$purvis = 0 if ($defaultRooting == 0);	# Safety; Purvis coding only makes sense in a rooted context

# Read in tree descriptions
	my @treeList = treeRead($treeFile);

	my $NumTaxa = scalar(@allTaxa);

	die "ERROR: No trees read in from file $treeFile\n" if (not @treeList);

# Parse tree descriptions
	print "\nParsing tree descriptions ...\n" if ($verbose);
	my $treeCount = 0;
	my @supportST;
	my (%treeStart, %treeStop);
	my ($wtSum, $wtNum) = (0, 0);
	my (%wtPresent, @wtList, %elementWt, %wtSet, $WtList);
	my @CharState;
	my $NumChar = 0;
	
	foreach my $treeID (@treeList)	# Get structure of each tree
		{
		my $tree = $treeDescription{$treeID};
		my (%ParentClade, %CladeDescrip, %CladeSupport, %open, %start, %stop, %present);
		
		$treeCount++;
		print "\nTree: $treeCount\n" if ($verbose);
		my $totalClade = ($tree =~ tr/(//);
		next if ($totalClade == 1);	# Tree has only one (uninformative) clade; not coded in MRP
		my ($position, $CladeID, $CurrentClade) = (0, 0, 0);
		my $OldCurrent;
		
	#	$tree =~ s/:\d+\s?$//;
		
		while ($position < length($tree))
			{
			my $offset = 1;
			if (substr($tree, $position, 1) eq "(")		#New clade; go one level deeper
				{
				$CladeID++;
				$ParentClade{$CladeID} = $CurrentClade;
				$CurrentClade = $CladeID;
				$open{$CurrentClade} = 1;
				$start{$CladeID} = $position;
				}
			elsif (substr($tree, $position, 1) eq ")")	#Clade finished; clean up and go up one level
				{
				$open{$CurrentClade} = 0;
				$OldCurrent = $CurrentClade;
				$CurrentClade = $ParentClade{$OldCurrent};
				$stop{$OldCurrent} = $position;
				$CladeDescrip{$OldCurrent} = substr($tree, $start{$OldCurrent}, $stop{$OldCurrent}-$start{$OldCurrent}+1);
				$CladeDescrip{$OldCurrent} =~ s/:\d+\.?\d*//g;	# Remove branch length information
				}
			elsif (substr($tree, $position, 1) eq ":")	#Branch lengths present; record as nodal support
				{
				until (substr($tree, $position+$offset, 1) eq "(" or substr($tree, $position+$offset, 1) eq ")" or substr($tree, $position+$offset, 1) eq "," or $position+$offset >= length($tree))
					{
					$offset++;
					}
				if (substr($tree, $position-1, 1) eq ")")	# Support value pertains to a clade; store if desired
					{
					$CladeSupport{$OldCurrent} = substr($tree, $position+1, $offset-1) if ($support);
	#					$CladeSupport{$OldCurrent} = 1 if ($CladeSupport{$OldCurrent} == 0);
					if ($treeCount == 1 and $STsupport == 1)
						{
						push @supportST, substr($tree, $position+1, $offset-1);
	#					$supportST[$OldCurrent-2] = substr($tree, $position+1, $offset-1);
						}
					}
				$position += $offset-1;
				}
			elsif (substr($tree, $position, 1) eq ",")
				{
				# Do nothing
				}
			elsif (substr($tree, $position, 1) eq "[")	# Skip over embedded comment
				{
				until (substr($tree, $position+$offset, 1) eq "]" or $position+$offset >= length($tree))
					{
					$offset++;
					}
				$position += $offset-1;
				}
			else	# Determine membership of clade
				{
				my $prevChar = substr($tree, $position-1, 1);
				until (substr($tree, $position+$offset, 1) eq "(" or substr($tree, $position+$offset, 1) eq ")" or substr($tree, $position+$offset, 1) eq "," or substr($tree, $position+$offset, 1) eq ":" or $position+$offset >= length($tree))
					{
					$offset++;
					}
				unless ($prevChar eq ")")	# Text represents a node label and not a taxon
					{
					for (my $clade = 1; $clade <= $CladeID; $clade++)	# Assign taxon to any open clades
						{
						if ($open{$clade})
							{
							$present{substr($tree, $position, $offset)}{$clade} = 1;
							}
						}
					}
				$position += $offset-1;
				}
			$position++;
			}
			
		if ($verbose)
			{
			for (1..$totalClade)
				{
				print "\t$_\t$CladeDescrip{$_}\n";
				if ($_ > 1)
					{
					print "\t\tSupport: $CladeSupport{$_}\n" if (defined $CladeSupport{$_} and $support);
					}
				}
			}
		
		# MRP coding subroutine; only codes taxa on trees (i.e., no hypothetical outgroups)
			# Get initial clades and taxa for rooted and unrooted versions
				my $initClade = 2;
				my $initTaxon = 0;
				unless ($rootingStatus{$treeID}) # Check whether or not clade 2 is informative and set starting clade as appropriate
					{
					my $zeroCount = 0;
					for (my $TaxonCount = $initTaxon; $TaxonCount < $NumTaxa; $TaxonCount++)
						{
						if ($present{$allTaxa[$TaxonCount]}{1})	# Only check taxa present on tree
							{
							unless ($present{$allTaxa[$TaxonCount]}{2})
								{
								$zeroCount++;
								}
							}
						}
					if ($zeroCount <= 1)	# Clade is uninformative; do not code
						{
						$initClade = 3;
						}
					}
			# Get initial clades and taxa for rooted and unrooted versions
	#			my $initClade = 2;
	#			my $initTaxon;
	#			if ($rooted)
	#				{
	#				$initTaxon = 1;	# For rooted trees, taxon = 0 is MRP outgroup; automatically coded
	#				}
	#			else
	#				{
	#				$initTaxon = 0;
	#				# Check whether or not clade 2 is informative and set starting clade as appropriate
	#					my $zeroCount = 0;
	#					for (my $TaxonCount = $initTaxon; $TaxonCount < $NumTaxa; $TaxonCount++)
	#						{
	#						if ($present{$allTaxa[$TaxonCount]}{1})	# Only check taxa present on tree
	#							{
	#							unless ($present{$allTaxa[$TaxonCount]}{2})
	#								{
	#								$zeroCount++;
	#								}
	#							}
	#						}
	#					if ($zeroCount <= 1)	# Clade is uninformative; do not code
	#						{
	#						$initClade = 3;
	#						}
	#				}
	
			# Set charset boundaries and increment wtset (if applicable)
				my $numElement = $totalClade - $initClade + 1;
				$treeStart{$treeCount} = $NumChar + 1;
				$treeStop{$treeCount} = $treeStart{$treeCount} + $numElement - 1;
				$NumChar = $treeStop{$treeCount};
				
				if ($rootingStatus{$treeID})
					{
					$MRPoutgroup .= "0" x $numElement;
					}
				else
					{
					$MRPoutgroup .= "?" x $numElement;
					}
				
				if ($support and $SupportFlag)
					{
					for (my $clade = $initClade; $clade <= $totalClade; $clade++)
						{
						if (defined $CladeSupport{$clade})	# Store supports
							{
							$wtSum += $CladeSupport{$clade};
							$wtNum++;
							unless ($wtPresent{$CladeSupport{$clade}})	# Store unique support values;
								{
								$wtPresent{$CladeSupport{$clade}} = 1;
								push @wtList, $CladeSupport{$clade};
								}
							my $element = $clade - $initClade + $treeStart{$treeCount};	# Get corresponding element number for clade
							$elementWt{$element} = $CladeSupport{$clade};
							$wtSet{$CladeSupport{$clade}} .= $element." ";
							}
						}
					}
	
			# MRP coding subroutine; only code taxa on trees
				for (my $TaxonCount = $initTaxon; $TaxonCount < $NumTaxa; $TaxonCount++)
					{
					if ($present{$allTaxa[$TaxonCount]}{1})	# Taxon is present on this source tree; {$clade = 1} is basal clade of tree
						{
						for (my $CladeCount = $initClade; $CladeCount <= $totalClade; $CladeCount++)
							{
							if ($present{$allTaxa[$TaxonCount]}{$CladeCount})		# Taxon is present in focal clade
								{
								$CharState[$TaxonCount] .= "1";
								}
	#						elsif ($purvis and $rooted)
							elsif ($purvis and not $unrootedFlag)
								{
								if ($present{$allTaxa[$TaxonCount]}{$ParentClade{$CladeCount}})	# Taxon is present in sister group
									{
									$CharState[$TaxonCount] .= "0";
									}
								else
									{
									$CharState[$TaxonCount] .= "?";
									}
								}
							else
								{
								$CharState[$TaxonCount] .= "0";
								}
							}
						}
					else	# Taxon is missing from this source tree
						{
						$CharState[$TaxonCount] .= "?"x($totalClade-$initClade+1);
						}
				}
		}

# Derive overall wtset and fill in missing support values
	if ($support and $SupportFlag)
		{
		# Assign average support value to nodes without support values
			my $avgWt = int($wtSum/$wtNum*1000)/1000;	# Rounded to three decimal places
			for (1..$NumChar)	# Assign average support vlaue
				{
				$wtSet{$avgWt} .= $_." " if (not defined $elementWt{$_});
				}

			push @wtList, $avgWt unless ($wtPresent{$avgWt} or not $wtSet{$avgWt}); # Store average support value if unique and used

		# Produce short version of wtset
			@wtList = sort {$a <=> $b} @wtList;
			foreach my $weight (@wtList)
				{
				$wtSet{$weight} =~ s/\s+$//;
				my @wtArray = split(/\s+/, $wtSet{$weight});
				@wtArray = sort {$a <=> $b} @wtArray;
				
				my $cleanList;
				if (scalar(@wtArray) > 1)
					{
					my $count = 1;
					while ($count < scalar(@wtArray))
						{
						if ($wtArray[$count] - $wtArray[$count-1] == 1)
							{
							my $lower = $count - 1;
							while ($count < scalar(@wtArray) and $wtArray[$count] - $wtArray[$count-1] == 1)
								{
								$count++;
								}
							$cleanList .= $wtArray[$lower]."-";
							}
						else
							{
							$cleanList .= $wtArray[$count-1]." ";
							$count++;
							}
						}
					$cleanList .= pop(@wtArray);
					$cleanList =~ s/\s+$//;
					}
				else
					{
					$cleanList = $wtArray[0];
					}
				$WtList .= "$weight: $cleanList";
					$WtList =~ s/\s+$//;		# Strip (any) trailing white space
					$WtList .= ", ";
				}
			$WtList =~ s/,\s+$//;		# Strip (any) trailing white space
		}

#if ($rooted)	# Produce all-zero outgroup
#	{
#	$CharState[0] = "0" x $NumChar;
#	}

if ($MRPoutgroup =~ /0/)	# At least some trees are rooted
	{
	unshift @allTaxa, "MRPoutgroup";
	unshift @CharState, $MRPoutgroup;
	$NumTaxa = scalar(@allTaxa);
	}

# Produce nexus file in transposed format
	my @TransData;
	if ($transpose)
		{
		my $position = 0;
		for ($position = 0; $position < $NumChar; $position++)
			{
#			undef $TransData[$position];
			for (my $taxon = 0; $taxon < $NumTaxa; $taxon++)
				{
				$TransData[$position] .= substr($CharState[$taxon], $position, 1)
				}
			}
			$TransData[$position] .= "\n";
		}

open (fDATA, ">MRPmatrix.nex");		# Print nexus file
	print fDATA "#nexus\n\nbegin data;\n";
	print fDATA "\tdimensions ntax = $NumTaxa nchar = $NumChar;\n";
	if ($transpose)
		{
		print fDATA "\ttaxlabels\n";
		foreach my $taxon (@allTaxa)
			{
			print fDATA "\t\t$taxon\n";
			}
		print fDATA "\t\t;\n";
		}
	print fDATA "\tformat missing = ?";
	if ($transpose)
		{
		print fDATA " transpose";
		}
	print fDATA ";\n";
	print fDATA "\n\tmatrix\n\n";
	if ($transpose)
		{
		my ($currentTree, $currentNode) = (1, 1);
		for (1..$NumChar)
			{
			if ($_ > $treeStop{$currentTree})
				{
				$currentTree++;
				$currentNode = 1;
				}
			print fDATA "tree".$currentTree."_".$currentNode."\t$TransData[$_-1]\n";
			$currentNode++;
			}
		}
	else
		{
		for (0..$NumTaxa-1)
			{
			print fDATA "$allTaxa[$_]\t$CharState[$_]\n";
			}
		}
	print fDATA "\t;\nend;\n\n";
	unless ($transpose)
		{
		if ($SupportFlag and $support)
			{
			print fDATA "begin assumptions;\n";
			print fDATA "\twtset support = $WtList;\n";
			print fDATA "end;\n\n";
			}
		print fDATA "begin sets;\n";
		for (1..$treeCount)
			{
			next unless (defined $treeStart{$_} and defined $treeStop{$_});	# Tree has only one (uninformative) clade; not coded in MRP
			print fDATA "\tcharset ";
			if ($keepName)
				{
				print fDATA $treeName{"treeID_$_"}." = ".$treeStart{$_};
				}
			else
				{
				print fDATA "tree".$_." = ".$treeStart{$_};
				}
			if ($treeStart{$_} == $treeStop{$_})
				{
				print fDATA ";\n";
				}
			else
				{
				print fDATA "-".$treeStop{$_}.";\n";
				}
			}
		print fDATA "end;\n\n";
		}
	if ($STsupport and @supportST)
		{
		print fDATA "SuperSupport:";
		foreach my $value (@supportST)
			{
			printf fDATA " %3.3f", $value;
			}
		print fDATA "\n";
		}
close fDATA;

# Subroutines used in the script

sub treeRead	# Subroutine to read in tree descriptions and derive taxon names; accounts for any translation tables
	{
	my $treeFile = shift;
	my $treeNum = 0;

	setLineBreak($treeFile);
	open (TREES, "<$treeFile") or die "Cannot open nexus-formatted treefile $treeFile\n";
		my (@allTrees, $rawTreeDescrip, $treeDescrip, $treeFlag, $treeCount, $rooting);
		my (@taxonList, %taxonPresent);
		my $NumChar = 0;
		my (%TTable, $translation, $translateFlag, $maxTaxonID, $transText);
		
		while (<TREES>)
			{
			chomp;
			$_ =~ s/^\s+//;		# Strip (any) leading white space
			if ($_ =~ m/begin trees;/i)		# Clear variables for each tree block
				{
				undef %TTable;
				$translation = $translateFlag = $treeFlag = $maxTaxonID = $treeCount = 0;
				$treeBlock++;
				}
			if ($_ =~ m/translate/i)	# Read translation table of taxon names (if any)
				{
				$translateFlag = $translation = 1;
				$transText = "";
				print "Reading translation table ...\n" if ($verbose);
				}
			if ($translateFlag)
				{
				unless ($_ =~ m/translate/i)
					{
					$transText .= $_;
					if ($_ =~ m/;/)
						{
						$translateFlag = 0;
						$transText =~ s/;$//;	#Strip trailing semi-colon
						my @transEntries = split(/,/, $transText);
						foreach my $entry (@transEntries)
							{
							$entry =~ s/^\s+//;		#Strip (any) leading white space
							my ($taxonID, $taxonName) = split(/\s/,$entry);
							$maxTaxonID = $taxonID if ($taxonID > $maxTaxonID);
							$taxonName =~ s/\s+//g;
							$TTable{$taxonID} = $taxonName;
							$taxonPresent{$taxonName}++;
							}
						}
					}
				}
			if ($_ =~ /tree/i and $_ =~ /=/)
				{
				unless ($_ =~ /^>/)		#Ignore (any) commented lines
					{
					$rawTreeDescrip = $treeDescrip = "";
					unless ($block == 0 and $treeBlock > 1)
						{
						$treeFlag = 1;
						}
					}
				}
			if ($treeFlag)
				{
				$rawTreeDescrip .= $_;
				if ($_ =~ m/:/)
					{
					$SupportFlag = 1;
					}
				if ($_ =~ m/;/)
					{
					$treeCount++;
					print "\nTree block: $treeBlock\tTree number: $treeCount\n" if ($verbose);
					print "\tTranslating taxon names ...\n" if ($verbose and $translation);
					# Determine if trees are rooted or not (default for no information is not rooted)
						if ($rawTreeDescrip =~ /\[&R\]/)
							{
							$rooting = 1;
							}
						elsif ($rawTreeDescrip =~ /\[&U\]/)
							{
							$rooting = 0;
							$unrootedFlag = 1;
							}
						else
							{
							$rooting = $defaultRooting;
							$unrootedFlag = 1 if ($defaultRooting == 0);
							}
					# Remove any embedded comments
						if ($rawTreeDescrip =~ /\[/)
							{
							for (my $position = 0; $position < length($rawTreeDescrip); $position++)
								{
								if (substr($rawTreeDescrip, $position, 1) eq "[")
									{
									$position++ until (substr($rawTreeDescrip, $position, 1) eq "]");
									}
								else
									{
									$treeDescrip .= substr($rawTreeDescrip, $position, 1);
									}
								}
							}
						else
							{
							$treeDescrip = $rawTreeDescrip;
							}
					$treeFlag = 0;
					my $position = 0;
					until (substr($treeDescrip, $position, 1) eq "(")		# Avoid anything before tree description proper
						{
						$position++;
						}
					my $tree = substr($treeDescrip, $position);
						$tree =~ s/;\s*$//;
						$tree =~ s/\s+//g;
					my ($name) = (split /=/, $treeDescrip, 0);		# Get name of tree, if any
						$name =~ s/\s*\w*tree\s+//i;
						$name =~ s/\s*\*\s*//i;
						$name =~ s/\s+$//g;
#						$treeNames{$treeCount} = $name;
					my $openCount = ($tree =~ tr/(//);
					my $closedCount = ($tree =~ tr/)//);
					if ($openCount == $closedCount)		# Tree is balanced; process
						{
						$position = length($tree) - 1;
						while ($position >= 0)
							{
							my $IDlength = 1;
							if (substr($tree, $position, 1) eq "(" or substr($tree, $position, 1) eq ")" or substr($tree, $position, 1) eq "," or substr($tree, $position, 1) eq ":")		#Tree structure; skip to next character
								{
								$position--;
								}
							else	# Read in taxonID
								{
								until (substr($tree, $position, 1) eq "(" or substr($tree, $position, 1) eq ")" or substr($tree, $position, 1) eq "," or substr($tree, $position, 1) eq ":")
									{
									$position--;
									$IDlength++;
									}
								unless (substr($tree, $position, 1) eq ":")		# Avoid reading branch lengths as taxon names
									{
									my $readName = substr($tree, $position+1, $IDlength-1);
print "$readName\n\t$taxonPresent{$readName}\n";
									if ($translation and not defined $taxonPresent{$readName})	# Translate taxon names; latter condition is a safety for BioPerl convert module which adds but doesn't use translation block
										{
										if ($TTable{$readName})
											{
											substr($tree, $position+1, $IDlength-1) = $TTable{$readName};		#Translate name
											unless ($taxonPresent{$TTable{$readName}})
												{
												push @taxonList, $TTable{$readName};
												$taxonPresent{$TTable{$readName}} = 1;
												}
											}
										else
											{
											print "\t\tUnidentified taxon ".$readName." in tree #".$treeCount." of tree block #".$treeBlock.". Replaced with xxx.\n";
											substr($tree, $position+1, $IDlength-1) = "xxx";		#Translate name
											unless ($taxonPresent{"xxx"})
												{
												push @taxonList, "xxx";
												$taxonPresent{"xxx"} = 1;
												}
											}
										}
									else	# Enter name as read from tree, unless already present
										{
										$taxonPresent{$readName}++;
										push @taxonList, $readName if ($taxonPresent{$readName} == 1);
										}
									}
								}
							}
						$treeNum++;
							my $treeID = "treeID_$treeNum";
						
						push @allTrees, $treeID;
							$treeDescription{$treeID} = $tree;
							$rootingStatus{$treeID} = $rooting;
							$treeName{$treeID} = $name;
						print "\tTree description:\n\t\t$tree\n" if ($verbose);
						}
					else	#Tree is unbalanced; reject
						{
						print "\tTree #".$treeCount." of tree block #".$treeBlock." is unbalanced. Not processed.\n";
						}
					}
				}
			}
	close TREES;
	
	# Process taxon list
		@allTaxa = sort(duplStrip(@taxonList));
	
	return @allTrees;
	}

sub setLineBreak	# Check line breaks of input files and set input record separator accordingly
	{
	my $inFile = shift;
	$/ ="\n";
	open (IN, "<$inFile") or die "Cannot open $inFile to check form of line breaks.\n";
		while (<IN>)
			{
			if ($_ =~ /\r\n/)
				{
				print "\tDOS line breaks detected ...\n" if ($verbose);
				$/ ="\r\n";
				last;
				}
			elsif ($_ =~ /\r/)
				{
				print "\tMac line breaks detected ...\n" if ($verbose);
				$/ ="\r";
				last;
				}
			else
				{
				print "\tUnix line breaks detected ...\n" if ($verbose);
				$/ ="\n";
				last;
				}
			}
	close IN;
	}

sub duplStrip	# Subroutine to strip duplicate entries from an array
	{
	my @duplArray = @_;
	my %dupl;		# Remove duplicate taxa
		my @stripArray = grep(!$dupl{$_}++, @duplArray);
		@stripArray = sort {$a cmp $b} @stripArray;
	return @stripArray;
	}

# Version history
#
#	v1.2.1 (November 14, 2005)
#		- added GNU GPL statement
#		- added suggested citation for program
#		- uses treeRead module v1.0.1 (fixes minor bugs)
#		- checks that necessary input files exist before proceeding
#
#	v1.2 (March 15, 2005)
#		- tree reading now done via treeRead subroutine
#		- rooting of trees can now be specified on an individual basis, allowing semi-rooted coding
#		- minor bug fixes
#
#	v1.1.1a (April 29, 2004)
#		- added ability to automatically set line breaks for input files
#		- minor bug fixes
#
#	v1.1.1 (January 14, 2004)
#		- minor bug fixes
#
#	v1.1 (August 28, 2003)
#		- rewrote code slightly to "use strict"
#		- minor bug fixes
#
#	v1.0.3 (August 7, 2003)
#		- added option to produce globally rooted and unrooted matrix representations
#		- minor bug fixes
#
#	v1.0.2 (March 9, 2003)
#		- minor bug fixes
#
#	v1.0.1 (March 15, 2005)
#		- added option to retain tree names in file
#		- minor bug fixes
#
#	v1.0 (June 21, 2002)
#		- initial release
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# A copy of the GNU General Public License is available at
# http://www.gnu.org/copyleft/gpl.html or by writing to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Fifth Floor, Boston, MA, 02110-1301, USA.
